"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MessageContext = void 0;
const attachments_1 = require("../models/attachments");
const Message_1 = require("../models/Message");
const Utils_1 = require("../Utils");
const ChatContext_1 = require("./ChatContext");
class MessageContext extends Message_1.Message {
    constructor(api, source) {
        super(source);
        this.api = api;
        this.match = [];
        this.chat = new ChatContext_1.ChatContext(this.api, this.get("chat"));
    }
    pin(id = this.id, disableNotification = true) {
        return this.execute("pinChatMessage", {
            message_id: id,
            disable_notification: disableNotification
        });
    }
    unpin(id = this.id) {
        return this.execute("unpinChatMessage", {
            message_id: id
        });
    }
    unpinAll() {
        return this.execute("unpinAllChatMessages", {});
    }
    editText(text = this.text, params) {
        let isCaptionEdit = (params && params.message_id) || this.hasAttachments;
        if (isCaptionEdit)
            params["caption"] = text;
        else
            params["text"] = text;
        return this.execute(isCaptionEdit
            ? "editMessageCaption"
            : "editMessageText", params);
    }
    delete(msgId = this.id) {
        return this.execute("deleteMessage", {
            message_id: msgId
        });
    }
    replyMessage(params) {
        params = typeof params === "string"
            ? { text: params }
            : params;
        return this.reply("message", params);
    }
    sendMessage(params) {
        params = typeof params === "string"
            ? { text: params }
            : params;
        return this.send("message", params);
    }
    async attach(type, source, params = {}) {
        let sourceParams = {
            [type]: typeof source === "string" || !(source instanceof (attachments_1.Attachment))
                ? source
                : { file_id: source }
        };
        let response = await this.execute(`send${Utils_1.StringUtils.capitalizeFirst(type)}`, Object.assign(Object.assign({}, params), sourceParams));
        return new Message_1.Message(response.message);
    }
    reply(method, params) {
        params.reply_to_message_id = this.id;
        return this.send(method, params);
    }
    async send(method, params) {
        params.chat_id = this.chat.id;
        let response = await this.execute(`send${Utils_1.StringUtils.capitalizeFirst(method)}`, params);
        return new Message_1.Message(response.message);
    }
    async execute(method, params, apiMethod = "callMethod") {
        params.chat_id = this.chat.id;
        return (await this.api[apiMethod](method, params)).result;
    }
}
exports.MessageContext = MessageContext;
